@@ -18,12 +18,15 @@
 # <http://www.gnu.org/licenses/>.
 
 """Helpers to handle debhelper command line options."""
+from __future__ import absolute_import
 
 import os
 import warnings
 
 from optparse import OptionParser, SUPPRESS_HELP, OptionValueError
 
+from ._version import version
+
 
 class DebhelperOptionParser(OptionParser):
     """Special OptionParser for handling Debhelper options.
@@ -76,7 +79,7 @@ def _check_for_deprecated_options(
 
 def get_default_parser():
     usage = '%prog [options]'
-    parser = DebhelperOptionParser(usage, version='%prog 1.0')
+    parser = DebhelperOptionParser(usage, version='%prog ' + version)
     parser.add_option('-p', '--package', action='append',
                       help='act on the package named PACKAGE')
     parser.add_option('-N', '--no-package', action='append',
@@ -92,6 +95,9 @@ def get_default_parser():
                       help=('package to install before processing '
                             'requirements.txt.'),
                       default=[])
+    parser.add_option('--extras', action='append',
+                      help=('Activate extras of the main package.'),
+                      default=[])
     parser.add_option('--pip-tool', default='pip',
                       help="Executable that will be used to install "
                       "requirements after the preinstall stage. Usually "
@@ -137,7 +143,7 @@ def get_default_parser():
                       help="Override installation path suffix")
     parser.add_option('--requirements',
                       dest='requirements_filename',
-                      help='Specify the filename for requirementst.txt',
+                      help='Specify the filename for requirements.txt',
                       default='requirements.txt')
     parser.add_option('--setuptools-test',
                       dest='setuptools_test',
